/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2011
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	Ad.h
**  Abstract :	This file implements device driver for AD module.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0512_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2011/05/06
**  
*******************************************************************************
*/
#ifndef _MDAD_
#define _MDAD_
/*
*******************************************************************************
**	Register bit define
*******************************************************************************
*/
/* A/D converter mode register(ADM) R/W */
/* Initial value */
#define AD_ADM_INITIALVALUE		0x00
/* A/D conversion operation control(ADCS) */
#define AD_CONVERSION_ENABLE	0x80	/* enable A/D conversion operation control */
#define AD_CONVERSION_DISABLE	0x00	/* disable A/D conversion operation control */
/* A/D conversion time selection(FR2,FR1,FR0) */
#define AD_CONVERSION_TIME		0x38	/* A/D Conversion Time Selection */
#define AD_CONVERSION_TIME0		0x00	/* 264/fPRS(2.7V<=AVref<=5.5V)	480/fPRS(2.3V<=AVref<=2.7V) */
#define AD_CONVERSION_TIME1		0x08	/* 176/fPRS(2.7V<=AVref<=5.5V)	320/fPRS(2.3V<=AVref<=2.7V) */
#define AD_CONVERSION_TIME2		0x10	/* 132/fPRS(2.7V<=AVref<=5.5V)	240/fPRS(2.3V<=AVref<=2.7V) */
#define AD_CONVERSION_TIME3		0x18	/* 88/fPRS(2.7V<=AVref<=5.5V)	160/fPRS(2.3V<=AVref<=2.7V) */
#define AD_CONVERSION_TIME4		0x20	/* 66/fPRS(2.7V<=AVref<=5.5V)	120/fPRS(2.3V<=AVref<=2.7V)*/
#define AD_CONVERSION_TIME5		0x28	/* 44/fPRS(2.7V<=AVref<=5.5V)	80/fPRS(2.3V<=AVref<=2.7V) */
/* A/D conversion AVref range selection(LV1,LV0) */
#define AD_CONVERSION_AVREF		0x06		/* AVref range*/
#define AD_CONVERSION_AVREF_23v	0x02		/* 2.3 v < AVref < 2.7 v*/
#define AD_CONVERSION_AVREF_27v	0x00		/* 2.7 V < AVref < 5.5 V*/
/* A/D comparator operation control(ADCE) */
#define AD_COMPARATOR_ENABLE	0x01		/* enable Comparator operation control */
#define AD_COMPARATOR_DISABLE	0x00		/* disable Comparator operation control */
/* A/D port configuration register(ADPC) */
/* Initial value */
#define AD_ADPC_INITIALVALUE	0x00
/* Analog input /digital input switching(ADPC3,ADPC2,ADPC1,ADPC0) */
#define AD_ADPC					0x0F
#define AD_ADPC_8ANALOG			0x00		/* ANI0 - ANI7 */
#define AD_ADPC_7ANALOG			0x01		/* ANI1 - ANI7 */
#define AD_ADPC_6ANALOG			0x02		/* ANI2 - ANI7 */
#define AD_ADPC_5ANALOG			0x03		/* ANI3 - ANI7 */
#define AD_ADPC_4ANALOG			0x04		/* ANI4 - ANI7 */
#define AD_ADPC_3ANALOG			0x05		/* ANI5 - ANI7 */
#define AD_ADPC_2ANALOG			0x06		/* ANI6 - ANI7 */
#define AD_ADPC_1ANALOG			0x07		/* ANI7 - ANI7 */
#define AD_ADPC_0ANALOG			0x08		/* ANI0 - ANI7(all digital) */

/* Analog input channel specification register (ADS) */
/* Initial value */
#define AD_ADS_INITIALVALUE		0x00
/* Analog input channel specification(ADS2,ADS1,ADS0) */
#define AD_INPUT_CHANNEL		0x07
#define AD_INPUT_CHANNEL_0		0x00		/* ANI0 */
#define AD_INPUT_CHANNEL_1		0x01		/* ANI1 */
#define AD_INPUT_CHANNEL_2		0x02		/* ANI2 */
#define AD_INPUT_CHANNEL_3		0x03		/* ANI3 */
#define AD_INPUT_CHANNEL_4		0x04		/* ANI4 */
#define AD_INPUT_CHANNEL_5		0x05		/* ANI5 */
#define AD_INPUT_CHANNEL_6		0x06		/* ANI6 */
#define AD_INPUT_CHANNEL_7		0x07		/* ANI7 */


/*
*******************************************************************************
**  Macro define
*******************************************************************************
*/
enum ADChannel { ADCHANNEL0, ADCHANNEL1, ADCHANNEL2, ADCHANNEL3, ADCHANNEL4, ADCHANNEL5, ADCHANNEL6, ADCHANNEL7};


/*
*******************************************************************************
**  Function define
*******************************************************************************
*/
void AD_Init( void );
void AD_Start( void );
void AD_Read( USHORT*  buffer );
void AD_Stop( void );
void AD_ComparatorOn( void );
void AD_ComparatorOff( void );
void AD_SelectADChannel( enum ADChannel channel );
__interrupt void MD_INTAD( void );
/* Start user code for definition. Do not edit comment generated here */
/* End user code for definition. Do not edit comment generated here */
#endif
