/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2011
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	Timer.c
**  Abstract :	This file implements device driver for Timer module.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0512_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2011/05/06
**  
*******************************************************************************
*/

/*
*******************************************************************************
**  Include files
*******************************************************************************
*/
#include "macrodriver.h"
#include "Timer.h"
/* Start user code for include definition. Do not edit comment generated here */
/* End user code for include definition. Do not edit comment generated here */
#include "user_define.h"

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
/* Start user code for global definition. Do not edit comment generated here */
/* End user code for global definition. Do not edit comment generated here */


/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function initializes TM51.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void TM51_Init( void )
{
	TCE51 = 0;			/* TM51 counter disable */
	TMMK51	= 1;		/* INTTM51 interrupt disabled */
	TMIF51	= 0;		/* clear interrupt flag */
	TMPR51	= 0;		/* set high priority level */
	TCL51 = TM5_INTER_CLOCK_3;
	 /* TM51 interval mode */
	CR51 = TM51_CR51_VALUE;
	TMC51 = TM5_TMC5_INITIALVALUE | TM5_OPT_MODE_OTHER | TM5_OUTPUT_FF_ENABLE;	
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function starts TM51 counter.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void TM51_Start( void )
{
	TMMK51	= 0;		/* INTTM51 interrupt enabled */
	TCE51 = 1;			/* count operation start */
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function stops TM51 counter.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void TM51_Stop( void )
{
	TCE51 = 0;			/* TM51 counter disable */
	TMMK51	= 1;		/* INTTM51 interrupt disabled */
	TMIF51	= 0;		/* clear interrupt flag */
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function changes TM51 compare value.
**
**	Parameters:
**		value :	value for compare register
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void TM51_ChangeTimerCondition( UCHAR value )
{
	CR51 = value;
}

/* Start adding user code. Do not edit comment generated here */
/* End user code adding. Do not edit comment generated here */


