/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2011
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	 main.c
**  Abstract :	This file implements main function.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0512_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2011/05/06
**  
*******************************************************************************
*/

/*
*******************************************************************************
**  Include files
*******************************************************************************
*/
#include "macrodriver.h"
#include "Ad.h"
#include "Serial.h"
#include "Timer.h"
#include "Port.h"
#include "System.h"
/* Start user code for include definition. Do not edit comment generated here */
/* End user code for include definition. Do not edit comment generated here */
#include "user_define.h"

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
/* Start user code for global definition. Do not edit comment generated here */
/* End user code for global definition. Do not edit comment generated here */


/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function  implements main function.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
#define	LCD_CMD_WAIT		400
#define	LCD_BIT_WAIT		10
#define	PRESET_MAX			60
#define	VOL_COUNT_SET		10
#define	FREQ_MIN			7100
#define	FREQ_MAX			7650
#define FREQ_MANUAL_DIV		9		/* 2011/5/21 */


unsigned short	ad_lev_in;
unsigned short	vol_level;
unsigned short	vol_level_m;

unsigned int	khz_no;
unsigned int	khz_no_m;
unsigned int	out_volt_data;
unsigned int	test_tun_data;
unsigned int	adj;

unsigned int	freq_khz;
unsigned int	freq_khz_b;
unsigned int	freq_khz_sub;

unsigned char	mode;

unsigned int	khz_no_setmode;

unsigned char	eeprom_addr=0xa0;
unsigned char	eeprom_finish;
unsigned char	eeprom_write_data[3];
unsigned char	eeprom_read_data[1];

unsigned char	d_flg;
unsigned char	tun_level;

unsigned int	k_set[130];
unsigned char	k_ch[130];

unsigned char	preset_no;
unsigned int	preset[PRESET_MAX];
unsigned char	preset_no_setmode;

unsigned char	set_count;
unsigned char	tun_flg;


void	wait_time(unsigned int time)
{
	while(time!=0){
		time--;
	}
}
void	kyokuhatsu_init(void)
{
unsigned char i;

	for(i=0;i<127;i++){
		k_set[i]=0;
	}
}
void	k_ch_init(void)
{
unsigned char i;

	for(i=0;i<127;i++){
		k_ch[i]=0xff;
	}
}
/*----------  LCD MODULE ----------*/
void	lcd_db(unsigned char data)
{
	P3.0=data&0x01;
	P3.1=(data&0x02)>>1;
	P3.2=(data&0x04)>>2;
	P3.3=(data&0x08)>>3;
}
void	lcd_wait(void)
{
	wait_time(5000);
}
void	lcd_c_set(void)
{
	P13.0=0;
	P14.0=1;
	wait_time(LCD_BIT_WAIT);
	P14.0=0;
}
void	lcd_d_set(void)
{
	P13.0=1;
	P14.0=1;
	wait_time(LCD_BIT_WAIT);
	P14.0=0;
}
void	lcd_cmd(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_c_set();
	lcd_db(data&0x0f);
	lcd_c_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_data(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_d_set();
	lcd_db(data&0x0f);
	lcd_d_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_init(void)
{
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(2);
	lcd_c_set();
	lcd_wait();
	
	lcd_cmd(0x28);
	lcd_cmd(0x08);
	lcd_cmd(0x0c);
	lcd_cmd(0x06);
}
void	lcd_clear(void)
{
	lcd_cmd(0x01);
}
void	lcd_xy(unsigned char x,unsigned char y)
{
unsigned char data;

	data=0x80;
	switch (y){
		case 0:data=0x80;break;
		case 1:data=0xc0;break;
		case 2:data=0x90;break;
		case 3:data=0xd0;break;
	}
	data=data+x;
	lcd_cmd(data);
}
void	lcd_print(char *data)
{
	while (*data!=0){
		lcd_data(*data++);
	}
}
void	lcd_su(unsigned char data)
{
	switch (data){
		case 0:lcd_print("0");break;
		case 1:lcd_print("1");break;
		case 2:lcd_print("2");break;
		case 3:lcd_print("3");break;
		case 4:lcd_print("4");break;
		case 5:lcd_print("5");break;
		case 6:lcd_print("6");break;
		case 7:lcd_print("7");break;
		case 8:lcd_print("8");break;
		case 9:lcd_print("9");break;
		case 10:lcd_print("A");break;
		case 11:lcd_print("B");break;
		case 12:lcd_print("C");break;
		case 13:lcd_print("D");break;
		case 14:lcd_print("E");break;
		case 15:lcd_print("F");break;
	}
}
void	lcd_khz(unsigned int hz)
{
unsigned char n0,n1,n2,n3,n4;
unsigned int count;

	count=hz;
	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);count=(count-n3)/10;
	n4=(unsigned char)(count%10);

	if(n4==0){lcd_print(" ");}else{lcd_su(n4);}
	lcd_su(n3);
	lcd_su(n2);
	lcd_su(n1);
	lcd_su(n0);
}
void	lcd_pr_byte(unsigned char data)
{
unsigned char data2;

	data2=data%16;data=data/16;
	lcd_su(data);
	lcd_su(data2);
}
void	lcd_ini_disp(void)
{
	lcd_xy(4,0);
	lcd_print("     kHz");
}
void	lcd_1keta10(unsigned char count)
{
unsigned char n0;

	n0=(unsigned char)(count%10);

	lcd_su(n0);
}

void	lcd_2keta10(unsigned char count)
{
unsigned char n0,n1;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);

	lcd_su(n1);
	lcd_su(n0);
}
void	lcd_4keta10(unsigned int count)
{
unsigned char n0,n1,n2,n3;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);

	lcd_su(n3);
	lcd_su(n2);
	lcd_su(n1);
	lcd_su(n0);
}
void	tun_lcd_clr(void)
{
	/*lcd_xy(4,1);
	lcd_print("       ");*/
	lcd_xy(12,0);
	lcd_print("    ");
	d_flg=0;
	tun_level=0;
	lcd_xy(0,0);
	lcd_print("   ");
}
void	lcd_auto_scan(void)
{
	lcd_xy(5,1);
	lcd_print("  AUTO SCAN");
}
void	lcd_kyokumei_clr(void)
{
	lcd_xy(5,1);
	lcd_print("           ");
}
void	lcd_mode(unsigned char m)
{
	lcd_xy(0,1);
	switch(m){
		case 1:	lcd_print("TUNE");
				if(tun_flg==1){lcd_print("m");}else{lcd_print("a");}
				break;
		case 0:	lcd_print("CH-");lcd_2keta10(preset_no);break;
		case 2:	lcd_print("SET");lcd_2keta10(preset_no_setmode);break;
	}
}
void	lcd_ch_disp(void)
{
	if(k_ch[(khz_no-FREQ_MIN)/5]!=0xff){
		lcd_xy(8,1);
		lcd_print("(CH-  )");
		lcd_xy(12,1);
		lcd_2keta10(k_ch[(khz_no-FREQ_MIN)/5]);
	}
	else{
		lcd_xy(8,1);lcd_print("       ");
	}
}
/*---------- VOLTAGE  ----------*/
void	dac_out_ch1(unsigned int data)
{
unsigned int 	d0;
unsigned char	d1;
unsigned char	i;

	P7.3=0;
	P7.4=1;
	d0=data << 4;
	d1=0x20;
	P7.4=0;
	
	for(i=0;i<8;i++){
		if((d1 & 0x80)==0x80){
			P7.5=1;
		}
		else{
			P7.5=0;
		}
		P7.3=1;
		d1=d1 << 1;
		P7.3=0;
	}
	
	for(i=0;i<16;i++){
		if((d0 & 0x8000)==0x8000){
			P7.5=1;
		}
		else{
			P7.5=0;
		}
		P7.3=1;
		d0=d0 << 1;
		P7.3=0;
	}
	P7.4=1;
	P7.5=0;
}
void	dac_out_ch2(unsigned int data)
{
unsigned int 	d0;
unsigned char	d1;
unsigned char	i;

	P7.3=0;
	P7.4=1;
	d0=data << 4;
	d1=0x21;
	P7.4=0;
	
	for(i=0;i<8;i++){
		if((d1 & 0x80)==0x80){
			P7.5=1;
		}
		else{
			P7.5=0;
		}
		P7.3=1;
		d1=d1 << 1;
		P7.3=0;
	}
	
	for(i=0;i<16;i++){
		if((d0 & 0x8000)==0x8000){
			P7.5=1;
		}
		else{
			P7.5=0;
		}
		P7.3=1;
		d0=d0 << 1;
		P7.3=0;
	}
	P7.4=1;
	P7.5=0;
}


void	vol_out1(unsigned int volt)
{
	dac_out_ch1(volt);		/* DAC1 CH A */
}
void	volt_up1(void)
{
	out_volt_data++;
	if(out_volt_data>=4096){
		out_volt_data=4095;
	}
}
void	volt_down1(void)
{
	if(out_volt_data>0){
		out_volt_data--;
	}
}
void	vol_out_tun1(unsigned int volt)
{
	dac_out_ch2(volt);		/* DAC1 CH B */
}

void	vol_out_sub(unsigned int volt)
{
	vol_out1(volt);
}
void	vol_out_tun_sub(unsigned int volt)
{
	vol_out_tun1(volt);
}

/*----------  KHZ    ----------*/
void	khz_inc_5(void)
{
	khz_no=khz_no+5;
	if(khz_no>FREQ_MAX){khz_no=FREQ_MIN;}
}

void	khz_dec_5(void)
{
	khz_no=khz_no-5;
	if(khz_no<FREQ_MIN){khz_no=FREQ_MAX;}
}
void	khz_inc_50(void)
{
	khz_no=khz_no+50;
	if(khz_no>FREQ_MAX){khz_no=FREQ_MIN;}
}

void	khz_dec_50(void)
{
	khz_no=khz_no-50;
	if(khz_no<FREQ_MIN){khz_no=FREQ_MAX;}
}

void	khz_set(void)
{
unsigned long data;

unsigned int n;

	n=(khz_no-FREQ_MIN)/5;
	if(k_set[n]==0){
		data=(unsigned long)khz_no;
		
		data=data-FREQ_MIN;
		data=data*8;
		/*
		if((khz_no>=9370)&&(khz_no<9500)){data=(85615*(data-9370)/10000+145);}
		if((khz_no>=9500)&&(khz_no<9600)){data=(68300*(data-9500)/10000+1258);}
		if((khz_no>=9600)&&(khz_no<9700)){data=(34900*(data-9600)/10000+1941);}
		if((khz_no>=9700)&&(khz_no<9800)){data=(40700*(data-9700)/10000+2290);}
		if((khz_no>=9800)&&(khz_no<9900)){data=(49000*(data-9800)/10000+2697);}
		if((khz_no>=9900)&&(khz_no<=10000)){data=(57600*(data-9900)/10000+3187);}
		*/
	}
	else{
		data=(unsigned long)k_set[n];
	}

	out_volt_data=(unsigned int)data;
	vol_out_sub(out_volt_data);
	/*lcd_xy(12,1);lcd_4keta10(out_volt_data);*/
}

/*
void	khz_set_direct(unsigned int data)
{

	out_volt_data=data;
	vol_out_sub(out_volt_data);
	lcd_xy(12,1);lcd_4keta10(out_volt_data);
}
*/

void	khz_set_tun(void)
{
unsigned long data;

	data=(unsigned long)khz_no;

	/*data=data-FREQ_MIN;
	data=data*8;*/
	
	
	if((khz_no>=7100)&&(khz_no<7150)){data=(13653*(data-7100)/10000+515);}
	if((khz_no>=7150)&&(khz_no<7200)){data=(19859*(data-7150)/10000+583);}
	if((khz_no>=7200)&&(khz_no<7250)){data=(22341*(data-7200)/10000+683);}
	if((khz_no>=7250)&&(khz_no<7300)){data=(24824*(data-7250)/10000+794);}
	if((khz_no>=7300)&&(khz_no<7350)){data=(31030*(data-7300)/10000+918);}
	if((khz_no>=7350)&&(khz_no<7400)){data=(32271*(data-7350)/10000+1074);}
	if((khz_no>=7400)&&(khz_no<7450)){data=(33512*(data-7400)/10000+1235);}
	if((khz_no>=7450)&&(khz_no<7500)){data=(32271*(data-7450)/10000+1403);}
	if((khz_no>=7500)&&(khz_no<7550)){data=(38477*(data-7500)/10000+1564);}
	if((khz_no>=7550)&&(khz_no<7600)){data=(35995*(data-7550)/10000+1756);}
	if((khz_no>=7600)&&(khz_no<=7650)){data=(44683*(data-7600)/10000+1936);}


	test_tun_data=(unsigned int)data;
	vol_out_tun_sub(test_tun_data);
}
/*----------  A/D ----------*/
void	ad_input7(void)
{
    WTMK = 1;			/* INTWT Interrupt servicing disabled */
	AD_SelectADChannel(AD_INPUT_CHANNEL_7);
	AD_Read(&ad_lev_in);
    WTMK = 0;			/* INTWT Interrupt servicing enabled */
	
}
unsigned int	vol_yomi(void)
{
unsigned short	sa;

	ad_input7();
	vol_level=ad_lev_in;
	if(vol_level>vol_level_m){
		sa=vol_level-vol_level_m;
	}
	if(vol_level<=vol_level_m){
		sa=vol_level_m-vol_level;
	}
	if(sa>=4){
		vol_level_m=vol_level;
	}
	else{
		vol_level=vol_level_m;
	}
	vol_level=vol_level/FREQ_MANUAL_DIV;
	vol_level=vol_level*5+FREQ_MIN;
		
	
	return vol_level;
}

/*----------- DATA INPUT ----------*/
void	freq_count_in(void)
{
unsigned long freq1;
unsigned int d0,d1,d2,d3,d4,d5;
unsigned char port2,port6;
unsigned int	i;

	P7.2=1;
	i=0;				/* wait */
	P7.0=1;P7.1=0;
	i=0			;		/* wait */
	port2=P2;port6=P6;
	d4=((port2 & 0x70) >> 4) | (port6 & 0x08);
	d5=(port2 & 0x0f);
	
	P7.0=0;P7.1=1;
	i=0;				/* wait */
	port2=P2;port6=P6;
	d2=((port2 & 0x70) >> 4) | (port6 & 0x08);
	d3=(port2 & 0x0f);

	P7.0=1;P7.1=1;
	i=0;				/* wait */
	port2=P2;port6=P6;
	d0=((port2 & 0x70) >> 4) | (port6 & 0x08);
	d1=(port2 & 0x0f);

	freq1=(unsigned long)d0+(unsigned long)d1*10+(unsigned long)d2*100+(unsigned long)d3*1000+(unsigned long)d4*10000+(unsigned long)d5*100000;

	freq_khz_sub=(unsigned int)(freq1%10);
	freq_khz=(unsigned int)((freq1-freq_khz_sub)/10);
	
	P7.2=0;

}
/*---------- MUTE -----------*/
void	mute_on(void)
{
	P12.1=0;
}
void	mute_off(void)
{
	P12.1=1;
}
/*---------- PRESET ---------*/
unsigned char	preset_next(unsigned char no)
{
unsigned char i,ret;

	ret=no;
	no++;
	if(no>=PRESET_MAX){no=0;}
	for(i=no;i<PRESET_MAX;i++){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	for(i=0;i<no;i++){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	return ret;
}

unsigned char	preset_back(unsigned char no)
{
unsigned char i,ret;

	ret=no;
	if(no==0){no=PRESET_MAX-1;}else{no--;}
	for(i=no;i>=0;i--){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	for(i=PRESET_MAX-1;i>=0;i--){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	return ret;
}

/*---------- EEPROM ---------*/
void	eeprom_write(unsigned char ad1,unsigned char ad2,unsigned char d)
{
	eeprom_write_data[0]=ad1;
	eeprom_write_data[1]=ad2;
	eeprom_write_data[2]=d;
	eeprom_finish = 0 ;
	IIC0_MasterSendStart(eeprom_addr, eeprom_write_data, 3,1);
	while (!eeprom_finish){};
}
unsigned char	eeprom_read(unsigned char ad1,unsigned char ad2)
{
unsigned char	data;

	eeprom_write_data[0]=ad1;
	eeprom_write_data[1]=ad2;
	eeprom_finish = 0 ;
	IIC0_MasterSendStart(eeprom_addr, eeprom_write_data, 2,1);
	while (!eeprom_finish){};

	eeprom_finish = 0 ;
	IIC0_MasterReceiveStart(eeprom_addr, eeprom_read_data,1,10);
	while (!eeprom_finish) {};
	
	data=eeprom_read_data[0];
	
	return data;
}
void	wait_eeprom(void)
{
		wait_time(1000);
}
void	eeprom_write_sub(void)
{
unsigned char	i;

	for(i=0;i<PRESET_MAX;i++){
		eeprom_write(0,0x10+i*2,(unsigned char)(preset[i]%256));wait_eeprom();
		eeprom_write(0,0x11+i*2,(unsigned char)(preset[i]/256));wait_eeprom();
	}
}
void	eeprom_write_kyokuhatsu(void)
{
unsigned char	i;

	for(i=0;i<127;i++){
		eeprom_write(2,0x00+i*2,(unsigned char)(k_set[i]%256));wait_eeprom();
		eeprom_write(2,0x01+i*2,(unsigned char)(k_set[i]/256));wait_eeprom();
	}
}

void	eeprom_init(void)
{
unsigned char	i;

		preset[0]=FREQ_MAX;

		for(i=1;i<PRESET_MAX;i++){
			preset[i]=0;
		}
		
		khz_no=FREQ_MIN;
		
		kyokuhatsu_init();

		eeprom_write(0,0,0xa5);wait_eeprom();
		
		eeprom_write_sub();

		eeprom_write_kyokuhatsu();

		eeprom_write(0,0x0a,(unsigned char)(khz_no%256));wait_eeprom();
		eeprom_write(0,0x0b,(unsigned char)(khz_no/256));wait_eeprom();
		eeprom_write(0,0x08,preset_no);wait_eeprom();
		eeprom_write(0,0x05,mode);wait_eeprom();
}

void	eeprom_check(void)
{
unsigned int sub0,sub1;
unsigned char	i;

	if(eeprom_read(0,0)==0xa5){
		for(i=0;i<PRESET_MAX;i++){
			sub0=(unsigned int)eeprom_read(0,0x10+i*2);wait_eeprom();
			sub1=(unsigned int)eeprom_read(0,0x11+i*2);wait_eeprom();
			preset[i]=sub0+sub1*256;
		}
		
		for(i=0;i<127;i++){
			sub0=(unsigned int)eeprom_read(2,0x00+i*2);wait_eeprom();
			sub1=(unsigned int)eeprom_read(2,0x01+i*2);wait_eeprom();
			k_set[i]=sub0+sub1*256;
		}
		

		sub0=(unsigned int)eeprom_read(0,0x0a);wait_eeprom();
		sub1=(unsigned int)eeprom_read(0,0x0b);wait_eeprom();
		khz_no=sub0+sub1*256;
		preset_no=eeprom_read(0,0x08);wait_eeprom();
		mode=eeprom_read(0,0x05);wait_eeprom();
	}
	else{
		eeprom_init();
	}
}
void	write_direct(void){

		eeprom_write(0,0x0a,(unsigned char)(khz_no%256));wait_eeprom();
		eeprom_write(0,0x0b,(unsigned char)(khz_no/256));wait_eeprom();
		eeprom_write(0,0x08,preset_no);wait_eeprom();
		eeprom_write(0,0x05,mode);wait_eeprom();
}
void	write_direct_set(unsigned char	no){

		eeprom_write(0,0x10+no*2,(unsigned char)(preset[no]%256));wait_eeprom();
		eeprom_write(0,0x11+no*2,(unsigned char)(preset[no]/256));wait_eeprom();
}
void	write_direct_kyokuhatsu_set(unsigned char	no){

		eeprom_write(2,0x00+no*2,(unsigned char)(k_set[no]%256));wait_eeprom();
		eeprom_write(2,0x01+no*2,(unsigned char)(k_set[no]/256));wait_eeprom();
}
/*---------------------------*/
void	k_ch_set(void)
{
unsigned char	i;
unsigned int	khz;

	for(i=0;i<PRESET_MAX;i++){
		if(preset[i]!=0x00){
			khz=preset[i];
			k_ch[(khz-FREQ_MIN)/5]=i;
		}
	}
}
/*---------- signal out ----------*/
void	head_out(void)
{
	set_count=4;
	TM51_Start();P4.1=1;
	while(set_count>0){}
	TM51_Stop();P4.1=0;
	set_count=4;
	TM51_Start();
	while(set_count>0){}
	TM51_Stop();
}
void	data1_out(void)
{
	set_count=1;
	TM51_Start();P4.1=1;
	while(set_count>0){}
	TM51_Stop();P4.1=0;
	set_count=3;
	TM51_Start();
	while(set_count>0){}
	TM51_Stop();
}
void	data0_out(void)
{
	set_count=1;
	TM51_Start();P4.1=1;
	while(set_count>0){}
	TM51_Stop();P4.1=0;
	set_count=1;
	TM51_Start();
	while(set_count>0){}
	TM51_Stop();
}
void	end_out(void)
{
	set_count=1;
	TM51_Start();P4.1=1;
	while(set_count>0){}
	TM51_Stop();P4.1=0;
	set_count=35;
	TM51_Start();
	while(set_count>0){}
	TM51_Stop();
}
void	data8_out(unsigned int data)
{
unsigned int d;
unsigned char i;

	for(i=0;i<8;i++){
		d=data & 0x01;
		if(d==0x01){data1_out();}
		else{data0_out();}
		data=data>>1;
	}
}
void	khz_data_out(unsigned int khz)
{
unsigned int n1,n2;

	n1=khz%256;
	n2=khz/256;
	head_out();data8_out(n1);data8_out(n2);end_out();
}

/*---------- MAIN -----------*/
void	main( void )
{
unsigned int	sub;
unsigned char	sw1_key;
unsigned char	sw1_rep;
unsigned char	sw1_cnt;
unsigned char	sw2_key;
unsigned char	sw2_rep;
unsigned char	sw2_cnt;
unsigned char	sw3_key;
unsigned char	sw3_rep;
unsigned char	sw3_cnt;
unsigned char	sw3_off;
unsigned char	sw4_key;
unsigned char	sw4_rep;
unsigned char	sw4_cnt;

unsigned int	nn;

unsigned char	auto_scan;
unsigned char	scan_flg;

unsigned int	vol_count;

	out_volt_data=0;

	sw1_key=0;sw1_rep=0;sw1_cnt=0;
	sw2_key=0;sw2_rep=0;sw2_cnt=0;
	sw3_key=0;sw3_rep=0;sw3_cnt=0;sw3_off=0;
	sw4_key=0;sw4_rep=0;sw4_cnt=0;
	auto_scan=0;scan_flg=0;
	mode=1;preset_no=0;

	vol_count=VOL_COUNT_SET;
	
	mute_on();

	AD_Start();
	
	lcd_init();
	lcd_clear();

	P12.2=1;P4.0=1;
	lcd_xy(0,1);
	lcd_print("SW 41mb RECEIVER");
	wait_time(50000);

	if(P1.0==0){auto_scan=1;}

	if((P1.1==0)&(P1.2==0)){
		eeprom_init();
	}
	else{
		eeprom_check();
	}

	lcd_ini_disp();
	if(auto_scan==1){lcd_auto_scan();}
	
	while(P1.3==0){}
	while(P1.0==0){}
	wait_time(10000);
	
	freq_khz_b=0;
	adj=0;sub=0;
	d_flg=0;tun_level=0;
	tun_flg=0;

	if(auto_scan==1){khz_no=FREQ_MIN;mode=1;}
	
	k_ch_init();
	k_ch_set();

	switch(mode){
		case 0:
				khz_no=preset[preset_no];
				khz_set();
				khz_set_tun();
				lcd_xy(4,0);
				lcd_khz(khz_no);
				adj=0;d_flg=0;
				tun_lcd_clr();
				lcd_kyokumei_clr();
				lcd_mode(mode);
				khz_data_out(khz_no);
				break;
		case 1:	if(tun_flg==1){
					khz_no=vol_yomi();
				}
				khz_no_m=khz_no;
				khz_set();
				khz_set_tun();
				lcd_xy(4,0);
				lcd_khz(khz_no);
				adj=0;d_flg=0;
				tun_lcd_clr();
				if(auto_scan==1){}else{lcd_kyokumei_clr();lcd_ch_disp();}
				lcd_mode(mode);
				khz_data_out(khz_no);
				break;
	}

	while(P1.3==0){}


	/* Start user code. Do not edit comment generated here */
	while (1) {
	/* key scan 1 */
		if (P1.0==0){
				sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if (P1.1==0){
				sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if (P1.2==0){
				sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			/*sw3_cnt=0;*/
		}
		/*------------------*/
		if (P1.3==0){
				sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/


		if(freq_khz!=freq_khz_b){
			freq_khz_b=freq_khz;
		}
		
		/*wait_time(2000);*/
		wait_time(1000);
		


		if(freq_khz>(khz_no+3)){adj=0;}
		if(freq_khz<(khz_no-3)){adj=0;}

		if(adj==0){
		/*--------------------*/
		freq_count_in();
		
		nn=(khz_no-FREQ_MIN)/5;
		

		if(freq_khz==khz_no){
			switch(freq_khz_sub){
				case 9:
				case 8:
				case 7:
				case 6:
				case 5:
				case 4:
				case 3:
						volt_down1();
						vol_out_sub(out_volt_data);
						khz_set_tun();
						adj=0;d_flg=1;tun_level=0;
						break;
				case 2:	scan_flg=1;mute_off();tun_level=1;adj=150;d_flg=1;k_set[nn]=out_volt_data;
						/*vol_out_sub(out_volt_data);
						khz_set_tun();*/
						break;
				case 1:	scan_flg=1;mute_off();tun_level=2;adj=150;d_flg=1;k_set[nn]=out_volt_data;
						/*vol_out_sub(out_volt_data);
						khz_set_tun();*/
						break;
				case 0:	scan_flg=1;mute_off();tun_level=3;adj=150;d_flg=1;k_set[nn]=out_volt_data;
						/*vol_out_sub(out_volt_data);
						khz_set_tun();*/
						break;
			}
		}
		if(freq_khz==(khz_no+1)){
						volt_down1();
						vol_out_sub(out_volt_data);
						khz_set_tun();
						adj=0;d_flg=0;tun_level=0;
		}
		if(freq_khz==(khz_no-1)){
			switch(freq_khz_sub){
				case 9:	scan_flg=1;mute_off();tun_level=2;adj=150;d_flg=1;k_set[nn]=out_volt_data;
						/*vol_out_sub(out_volt_data);
						khz_set_tun();*/
						break;
				case 8:	scan_flg=1;mute_off();tun_level=1;adj=150;d_flg=1;k_set[nn]=out_volt_data;
						/*vol_out_sub(out_volt_data);
						khz_set_tun();*/
						break;
				case 7:
				case 6:
				case 5:
				case 4:
				case 3:
				case 2:
				case 1:
				case 0:
						volt_up1();
						vol_out_sub(out_volt_data);
						khz_set_tun();
						adj=0;d_flg=1;tun_level=0;
						break;
			}
		}
		if(freq_khz>(khz_no+1)){
						volt_down1();
						vol_out_sub(out_volt_data);
						khz_set_tun();
						adj=0;d_flg=0;tun_level=0;
		}
		if(freq_khz<(khz_no-1)){
						volt_up1();
						vol_out_sub(out_volt_data);
						khz_set_tun();
						adj=0;d_flg=0;tun_level=0;
		}
		
		if(d_flg==1){
			/*--------------
			lcd_xy(4,1);
			lcd_khz(freq_khz);
			lcd_xy(9,1);
			lcd_print(".");
			lcd_xy(10,1);
			lcd_1keta10((unsigned char)freq_khz_sub);
			--------------*/
			if(auto_scan==0){
				lcd_xy(0,0);
				switch(tun_level){
					case 0:lcd_print("   ");break;
					case 1:lcd_print("*  ");break;
					case 2:lcd_print("** ");break;
					case 3:lcd_print("***");break;
				}
				lcd_xy(12,0);lcd_4keta10(out_volt_data);
			}
		}

			/*-------------------------------
			lcd_xy(4,1);
			lcd_khz(freq_khz);
			lcd_xy(9,1);
			lcd_print(".");
			lcd_xy(10,1);
			lcd_1keta10((unsigned char)freq_khz_sub);
			lcd_xy(12,0);lcd_4keta10(out_volt_data);
			-------------------------------*/
		

		/*-----------------------*/
		}
		else{
			adj--;
		}
		/*-------------------*/

	/* key scan 2 */
		if ((P1.0==0)&&(sw1_key==0)){
			sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if ((P1.1==0)&&(sw2_key==0)){
			sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if ((P1.2==0)&&(sw3_key==0)){
			sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			/*sw3_cnt=0;*/
			if(sw3_cnt>0){
				sw3_off=1;
			}
		}
		/*------------------*/
		if ((P1.3==0)&&(sw4_key==0)){
			sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/


	/* key action */
	
		if((sw1_key==0)&&(sw1_rep==0)){
			switch(mode){
				case 0:
						sw1_rep=1;
						mute_on();
						preset_no=preset_next(preset_no);
						khz_no=preset[preset_no];
						khz_set();
						khz_set_tun();
						lcd_xy(4,0);
						lcd_khz(khz_no);
						adj=0;d_flg=0;
						sw1_cnt=0;
						tun_lcd_clr();
						auto_scan=0;
						lcd_kyokumei_clr();
						lcd_mode(mode);
						khz_data_out(khz_no);
						write_direct();
						/*lcd_xy(12,1);lcd_4keta10(out_volt_data);*/
						break;
				case 1:	
						sw1_rep=1;
						if(tun_flg==0){
						mute_on();
						khz_inc_5();
						khz_set();
						khz_set_tun();
						lcd_xy(4,0);
						lcd_khz(khz_no);
						khz_no_m=khz_no;
						adj=0;d_flg=0;
						sw1_cnt=0;
						tun_lcd_clr();
						auto_scan=0;
						lcd_kyokumei_clr();
						lcd_ch_disp();
						lcd_mode(mode);
						khz_data_out(khz_no);
						write_direct();
						/*lcd_xy(12,1);lcd_4keta10(out_volt_data);*/
						}
						break;
				case 2:
						sw1_rep=1;
						preset_no_setmode++;
						if(preset_no_setmode>=PRESET_MAX){preset_no_setmode=1;}
						khz_no_setmode=preset[preset_no_setmode];
						lcd_xy(8,1);
						if(khz_no_setmode!=0){
							lcd_khz(khz_no_setmode);
							lcd_print("kHz");
						}
						else{
							lcd_print("=NONE=  ");
						}
						adj=0;d_flg=0;
						sw1_cnt=0;
						tun_lcd_clr();
						auto_scan=0;
						lcd_mode(mode);
						break;
			}
		}
		/*------------------*/
		if((sw1_key==0)&&(sw1_rep==1)){
			sw1_cnt++;
			if(sw1_cnt>=30){
				switch(mode){
					case 0:	break;
					case 1:
							mute_on();
							khz_inc_50();
							khz_set();
							khz_set_tun();
							lcd_xy(4,0);
							lcd_khz(khz_no);
							adj=0;d_flg=0;
							sw1_cnt=0;
							tun_lcd_clr();
							khz_data_out(khz_no);
							write_direct();
							break;
					case 2:	break;
				}
			}
			adj=0;
		}
		/*------------------*/
		if((sw2_key==0)&&(sw2_rep==0)){
			switch(mode){
				case 0:
						sw2_rep=1;
						mute_on();
						preset_no=preset_back(preset_no);
						khz_no=preset[preset_no];
						khz_set();
						khz_set_tun();
						lcd_xy(4,0);
						lcd_khz(khz_no);
						adj=0;d_flg=0;
						sw2_cnt=0;
						tun_lcd_clr();
						auto_scan=0;
						lcd_kyokumei_clr();
						lcd_mode(mode);
						khz_data_out(khz_no);
						write_direct();
						/*lcd_xy(12,1);lcd_4keta10(out_volt_data);*/
						break;
				case 1:
						sw2_rep=1;
						if(tun_flg==0){
						mute_on();
						khz_dec_5();
						khz_set();
						khz_set_tun();
						lcd_xy(4,0);
						lcd_khz(khz_no);
						khz_no_m=khz_no;
						adj=0;d_flg=0;
						sw2_cnt=0;
						tun_lcd_clr();
						auto_scan=0;
						lcd_kyokumei_clr();
						lcd_ch_disp();
						lcd_mode(mode);
						khz_data_out(khz_no);
						write_direct();
						/*lcd_xy(12,1);lcd_4keta10(out_volt_data);*/
						}
						break;
				case 2:
						sw2_rep=1;
						if(preset_no_setmode<=1){preset_no_setmode=PRESET_MAX-1;}
						else{preset_no_setmode--;}
						khz_no_setmode=preset[preset_no_setmode];
						lcd_xy(8,1);
						if(khz_no_setmode!=0){
							lcd_khz(khz_no_setmode);
							lcd_print("kHz");
						}
						else{
							lcd_print("=NONE=  ");
						}
						adj=0;d_flg=0;
						sw2_cnt=0;
						tun_lcd_clr();
						auto_scan=0;
						lcd_mode(mode);
						break;
			}
		}
		/*--------------------*/
		if((sw2_key==0)&&(sw2_rep==1)){
			sw2_cnt++;
			if(sw2_cnt>=30){
				switch(mode){
					case 0:	break;
					case 1:
							mute_on();
							khz_dec_50();
							khz_set();
							khz_set_tun();
							lcd_xy(4,0);
							lcd_khz(khz_no);
							adj=0;d_flg=0;
							sw2_cnt=0;
							tun_lcd_clr();
							khz_data_out(khz_no);
							write_direct();
							break;
					case 2:	break;
				}
			}
			adj=0;
		}
		/*--------------------*/
		if((sw3_key==0)&&(sw3_rep==0)){
			sw3_rep=1;
		}
		/*--------------------*/
		if((sw3_key==0)&&(sw3_rep==1)){
			sw3_cnt++;
		}
		/*--------------------*/
		if((sw3_key==1)&&(sw3_off==1)){
			if(sw3_cnt>=40){
				/**/
				switch(mode){
					case 0:	break;
					case 1:
							mode=2;
							preset_no_setmode=preset_no;
							khz_no_setmode=preset[preset_no];
							lcd_xy(8,1);
							lcd_khz(khz_no_setmode);
							lcd_print("kHz");
							adj=0;d_flg=0;
							sw1_cnt=0;
							tun_lcd_clr();
							auto_scan=0;
							lcd_mode(mode);
							break;
					case 2:
							if(preset_no_setmode!=0){
								mode=1;
								preset_no=preset_no_setmode;
								preset[preset_no]=0;
								k_ch[(khz_no-FREQ_MIN)/5]=0xff;
								if(tun_flg==1){
									khz_no=vol_yomi();
								}
								khz_set();
								khz_set_tun();
								lcd_xy(4,0);
								lcd_khz(khz_no);
								adj=0;d_flg=0;
								tun_lcd_clr();
								auto_scan=0;
								lcd_kyokumei_clr();
								lcd_ch_disp();
								lcd_mode(mode);
								khz_data_out(khz_no);
								write_direct();
								write_direct_set(preset_no);
								preset_no=preset_next(preset_no);
							}
							break;
				}
			}
			else{
				/*Z*/
				switch(mode){
					case 0:
							mode=1;
							if(tun_flg==1){
								khz_no=vol_yomi();
							}
							khz_set();
							khz_set_tun();
							lcd_xy(4,0);
							lcd_khz(khz_no);
							adj=0;d_flg=0;
							tun_lcd_clr();
							auto_scan=0;
							lcd_kyokumei_clr();
							lcd_ch_disp();
							lcd_mode(mode);
							khz_data_out(khz_no);
							write_direct();
							break;
					case 1:
							mode=0;
							khz_no=preset[preset_no];
							khz_set();
							khz_set_tun();
							lcd_xy(4,0);
							lcd_khz(khz_no);
							adj=0;d_flg=0;
							tun_lcd_clr();
							auto_scan=0;
							lcd_kyokumei_clr();
							lcd_mode(mode);
							khz_data_out(khz_no);
							write_direct();
							break;
					case 2:
							if(preset_no_setmode!=0){
								mode=0;
								preset_no=preset_no_setmode;
								preset[preset_no]=khz_no;
								k_ch[(khz_no-FREQ_MIN)/5]=preset_no;
								khz_set();
								khz_set_tun();
								lcd_xy(4,0);
								lcd_khz(khz_no);
								adj=0;d_flg=0;
								tun_lcd_clr();
								auto_scan=0;
								lcd_kyokumei_clr();
								lcd_mode(mode);
								write_direct();
								write_direct_set(preset_no);
							}
							break;
				}
			}
			
			sw3_off=0;
			sw3_cnt=0;
		}
		/*--------------------*/
		if((sw4_key==0)&&(sw4_rep==0)){
			switch(mode){
				case 0:	sw4_rep=1;
						break;
				case 1:	sw4_rep=1;
						switch(tun_flg){
							case 0:	tun_flg=1;
									khz_no=vol_yomi();
									break;
							case 1:	tun_flg=0;
									break;
						}
						khz_set();
						khz_set_tun();
						lcd_xy(4,0);
						lcd_khz(khz_no);
						khz_no_m=khz_no;
						adj=0;d_flg=0;
						tun_lcd_clr();
						lcd_kyokumei_clr();lcd_ch_disp();
						lcd_mode(mode);
						khz_data_out(khz_no);
						break;
				case 2:	sw4_rep=1;
						mode=1;
						khz_set();
						khz_set_tun();
						lcd_xy(4,0);
						lcd_khz(khz_no);
						khz_no_m=khz_no;
						adj=0;d_flg=0;
						tun_lcd_clr();
						auto_scan=0;
						lcd_kyokumei_clr();
						lcd_ch_disp();
						lcd_mode(mode);
						khz_data_out(khz_no);
						write_direct();
						break;
			}
		}
		/*------------------*/
		if((sw4_key==0)&&(sw4_rep==1)){
			sw4_cnt++;
			if(sw4_cnt>=30){
				switch(mode){
					case 0:
					case 1:
							lcd_xy(0,1);
							lcd_print("                ");
							lcd_xy(0,0);
							lcd_print("=== POWER OFF===");
							eeprom_write_kyokuhatsu();
							P12.2=0;
							while(P1.3==0){}
							break;
					case 2:	break;
				}
			}
			adj=0;
		}
		/*--------------------*/
		if(auto_scan==1){
			if(scan_flg==1){
				write_direct_kyokuhatsu_set((unsigned char)nn);
				khz_inc_5();
				khz_set();
				khz_set_tun();
				lcd_xy(4,0);
				lcd_khz(khz_no);
				khz_data_out(khz_no);
				adj=0;
				/*tun_lcd_clr();*/
				scan_flg=0;
			}
			if(khz_no==FREQ_MAX){
				auto_scan=0;
				lcd_kyokumei_clr();
				khz_data_out(khz_no);
			}

		}
		/*------------------ */
		if((mode==1)&(tun_flg==1)){
		/*----------*/
		if(vol_count>0){vol_count--;}
		if(vol_count==0){
			khz_no=vol_yomi();
			if(khz_no>FREQ_MAX){khz_no=FREQ_MAX;}
			if(khz_no_m!=khz_no){
				khz_set();
				khz_set_tun();
				lcd_xy(4,0);
				lcd_khz(khz_no);
				adj=0;d_flg=0;
				tun_lcd_clr();
				lcd_kyokumei_clr();lcd_ch_disp();
				lcd_mode(mode);
				khz_data_out(khz_no);
				khz_no_m=khz_no;
			}
			vol_count=VOL_COUNT_SET;
		}
		}
		

	}
	/* End user code. Do not edit comment generated here */
}

/* Start adding user code. Do not edit comment generated here */
/* End user code adding. Do not edit comment generated here */


